package info.textgrid.middleware;

import java.math.BigInteger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Map;
import java.util.TimeZone;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import org.apache.commons.logging.LogFactory;
import info.textgrid.middleware.oaipmh.ListMetadataFormatsType;
import info.textgrid.middleware.oaipmh.MetadataFormatType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

/**
 * <p>
 * OAIPMHUtillities is responsible to implement all necessary functions to implement the arguments
 * of a request.
 * </p>
 * 
 * @author Maximilian Brodhun, SUB Göttingen
 * @version 2019-10-29
 * @since 2014-01-29
 */
public class OAIPMHUtilities {

  // **
  // FINALS
  // **

  public static final String OAIPMH_NAMESPACE = "http://www.openarchives.org/OAI/2.0/";
  public static final String OAIPMH_SCHEMA_LOCATION =
      "http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd";
  public static final String OAIDC_NAMESPACE = "http://www.openarchives.org/OAI/2.0/oai_dc/";
  public static final String OAIDC_SCHEMA_LOCATION =
      "http://www.openarchives.org/OAI/2.0/oai_dc.xsd";
  public static final String DC_NAMESPACE = "http://purl.org/dc/elements/1.1/";
  public static final String DC_SCHEMA_LOCATION =
      "https://dublincore.org/schemas/xmls/simpledc20021212.xsd";
  public static final String METS_NAMESPACE = "http://www.loc.gov/METS/";
  public static final String METS_SCHEMA_LOCATION = "http://www.loc.gov/standards/mets/mets.xsd";

  // **
  // STATICS
  // **

  private static org.apache.commons.logging.Log log = LogFactory.getLog(OAIPMHImpl.class);

  
  public static boolean isOAIPMH_RequestArgument(String verb){
	String allowedArgument= "^[identifier,from,until,set,resumptionToken,metadataPrefix]*$";
	return verb.matches(allowedArgument);
  }
  
  
  /**
   * <p>
   * Since TextGrid just supports DC the only returned MetadataFormat is DublinCore.
   * </p>
   * 
   * @return
   */
  public MetadataFormatType setMetadataFormats() {

    MetadataFormatType tgDublinCore = new MetadataFormatType();
    tgDublinCore.setMetadataNamespace("http://www.openarchives.org/OAI/2.0/oai_dc/");
    tgDublinCore.setMetadataPrefix("oai_dc");
    tgDublinCore.setSchema("http://www.openarchives.org/OAI/2.0/oai_dc.xsd");

    return tgDublinCore;
  }

  /**
   * <p>
   * Producing the list of all metadata formats.
   * </p>
   * 
   * @return tgRepMetadaFormats
   */
  public ListMetadataFormatsType setMetadataFormatList() {

    ListMetadataFormatsType tgRepMetadataFormats = new ListMetadataFormatsType();
    MetadataFormatType tgDublinCore = setMetadataFormats();
    tgRepMetadataFormats.getMetadataFormat().add(tgDublinCore);

    return tgRepMetadataFormats;
  }

  /**
   * <p>
   * Calculating the current date and give it back as XMLGregorianCalendar.
   * </p>
   * 
   * @return now
   * @throws DatatypeConfigurationException
   */
  public static XMLGregorianCalendar getXMLGregorianCalendarNow()
      throws DatatypeConfigurationException {

    GregorianCalendar gregorianCalendar = new GregorianCalendar();
    DatatypeFactory datatypeFactory = DatatypeFactory.newInstance();
    XMLGregorianCalendar now = datatypeFactory.newXMLGregorianCalendar(gregorianCalendar);

    return now;
  }

  /**
   * <p>
   * Converting a given string representing a date value into the date format required for OAIPMH
   * and give it back as XMLGregorianCalendar.
   * </p>
   * 
   * @param originalDateTimeString
   * @return xmlCal
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  public static XMLGregorianCalendar convertDateFormat(String originalDateTimeString)
      throws ParseException, DatatypeConfigurationException {

    SimpleDateFormat tgItemTime = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.S");
    Date date = tgItemTime.parse(originalDateTimeString);
    SimpleDateFormat outFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
    outFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
    String output2 = outFormatter.format(date);
    XMLGregorianCalendar xmlCal = DatatypeFactory.newInstance().newXMLGregorianCalendar(output2);

    return xmlCal;
  }

  /**
   * <p>
   * Converting a given string representing a date value into the date format required for OAIPMH
   * and give it back as string.
   * </p>
   * 
   * @param originalDateTimeString
   * @return dateOutputAsString
   * @throws ParseException
   */
  public static String datestampAsString(String originalDateTimeString) throws ParseException {

    SimpleDateFormat tgItemTime = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.S");
    Date date = tgItemTime.parse(originalDateTimeString);
    SimpleDateFormat outFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
    outFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
    String dateOutputAsString = outFormatter.format(date);

    return dateOutputAsString;
  }

  /**
   * <p>
   * Returns a resumptionToken for a search, if necessary.
   * </p>
   * 
   * TODO Check if we really must put all these references in!
   * 
   * @param completeListSize
   * @param resumptionToken
   * @param cursorCollector
   * @param scrollID
   * @param searchResponseSize
   * @param i
   * @return A resumption token if applicable, null otherwise.
   */
  public static ResumptionTokenType getResumptionToken(final long completeListSize,
	      final String resumptionToken, Map<String, Integer> cursorCollector, final String scrollID,
	      final int searchResponseSize, final int i) {

	    log.info("Creating a ResumptionToken:\n " +
	        "CompleteListSize: " + completeListSize + "\n" +
	        "ResumptionToken: " + resumptionToken + "\n" +
	        "CursorCollector: " + cursorCollector + "\n" +
	        "ScrollID: " + scrollID + "\n" +
	        "SearchResponseSize: " + scrollID + "\n" +
	        "IterationStep: " + i);

	    int cursor;

	    // Three cases here:
	    // 1. Complete list size is > searchResponseSize and a token already is existing: we need to
	    // check hash map!
	    // 2. Complete list size is > searchResponseSize and we have no token: we do need one!
	    // 3. Complete list size is <= searchResponseSize (we do not need a token! do nothing!)

	    if (completeListSize > searchResponseSize) {
	      ResumptionTokenType resTokenForResponse = new ResumptionTokenType();

	      if (resumptionToken != null && cursorCollector.containsKey(resumptionToken)) {
	        cursor = cursorCollector.get(resumptionToken).intValue() + i;
	        if(cursor-searchResponseSize == 0) {
	        	cursor *= 2;
	        }
	        resTokenForResponse.setCursor(BigInteger.valueOf((long) cursor));
	        cursorCollector.put(scrollID, cursor);
	      } else {
	        resTokenForResponse.setCursor(BigInteger.valueOf(searchResponseSize));
	        cursorCollector.put(scrollID, searchResponseSize);        
	        cursor = searchResponseSize;
	        resTokenForResponse.setCursor(BigInteger.valueOf((long) cursor));
	        
	      }

	      // Set resumption token string if cursor is less then complete list size: More objects can be
	      // delivered!
	      if (cursor < completeListSize) {    	 
	        resTokenForResponse.setValue(scrollID);
	      }

	      // Remove resumption token string if cursor is bigger then or equal complete list size: This
	      // is the last response, no more objects are available!
	      if (cursor >= completeListSize) {
	    	//cursor =;
	        resTokenForResponse.setValue("");
	      }

	      // Set complete list size.
	      resTokenForResponse.setCompleteListSize(BigInteger.valueOf(completeListSize));

	      return resTokenForResponse;
	    }

	    return null;
	  }

}
